@@ -22,9 +22,7 @@ use self::{
     dispatcher::Dispatcher,
 };
 use crate::messaging::{
-    client::ClientMsg,
-    node::{Peer, RoutingMsg},
-    DstInfo, DstLocation, EndUser, Itinerary, MessageType, SectionAuthorityProvider, WireMsg,
+    node::Peer, DstLocation, EndUser, Itinerary, MessageType, SectionAuthorityProvider, WireMsg,
 };
 use crate::routing::core::{join_network, Core};
 use crate::routing::{
@@ -358,7 +356,7 @@ impl Routing {
     pub async fn send_message(
         &self,
         itinerary: Itinerary,
-        content: Bytes,
+        content: MessageType,
         additional_proof_chain_key: Option<bls::PublicKey>,
     ) -> Result<()> {
         if let DstLocation::EndUser(EndUser { socket_id, xorname }) = itinerary.dst {
@@ -374,7 +372,7 @@ impl Routing {
                 if let Some(socket_addr) = addr {
                     debug!("Sending client msg to {:?}", socket_addr);
                     return self
-                        .send_message_to_client(socket_addr, xorname, ClientMsg::from(content)?)
+                        .send_message_to_client(socket_addr, xorname, content)
                         .await;
                 } else {
                     debug!(
@@ -403,18 +401,17 @@ impl Routing {
         &self,
         recipient: SocketAddr,
         user_xorname: XorName,
-        message: ClientMsg,
+        mut message: MessageType,
     ) -> Result<()> {
+        message.update_dst_info(
+            Some(*self.section_chain().await.last_key()),
+            Some(user_xorname),
+        );
+
         let command = Command::SendMessage {
             recipients: vec![(user_xorname, recipient)],
             delivery_group_size: 1,
-            message: MessageType::Client {
-                msg: message,
-                dst_info: DstInfo {
-                    dst: user_xorname,
-                    dst_section_pk: *self.section_chain().await.last_key(),
-                },
-            },
+            message,
         };
         self.dispatcher.clone().handle_commands(command).await
     }
@@ -516,7 +513,7 @@ async fn handle_message(dispatcher: Arc<Dispatcher>, bytes: Bytes, sender: Socke
             let _ = task::spawn(dispatcher.handle_commands(command));
         }
         MessageType::Routing { msg, dst_info } => {
-            if let Err(err) = RoutingMsg::check_signature(&msg) {
+            if let Err(err) = msg.check_signature() {
                 error!(
                     "Discarding message received ({:?}) due to invalid signature: {:?}",
                     msg.id, err
